@@ -1,18 +1,18 @@
 import binascii
 import codecs
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 import datetime
 import os
 import time
-from typing import Dict, List, Tuple, TYPE_CHECKING, Optional
+from typing import List, TYPE_CHECKING, Optional, Dict
 
 import grpc
 from grpc._channel import _Rendezvous
 from grpc._channel import _InactiveRpcError
 from google.protobuf.json_format import MessageToDict
 
-import lndmanage.grpc_compiled.rpc_pb2 as lnd
-import lndmanage.grpc_compiled.rpc_pb2_grpc as lndrpc
+import lndmanage.grpc_compiled.lightning_pb2 as lnd
+import lndmanage.grpc_compiled.lightning_pb2_grpc as lndrpc
 import lndmanage.grpc_compiled.router_pb2 as lndrouter
 import lndmanage.grpc_compiled.router_pb2_grpc as lndrouterrpc
 import lndmanage.grpc_compiled.walletkit_pb2 as lndwalletkit
@@ -28,7 +28,7 @@ from lndmanage.lib.ln_utilities import (
     channel_unbalancedness_and_commit_fee
 )
 from lndmanage.lib.psbt import extract_psbt_inputs_outputs
-from lndmanage.lib.types import UTXO, AddressType
+from lndmanage.lib.data_types import UTXO, AddressType
 from lndmanage.lib.user import yes_no_question
 from lndmanage.lib.utilities import convert_dictionary_number_strings_to_ints
 from lndmanage import settings
@@ -65,23 +65,23 @@ class Node(object):
 
 
 class LndNode(Node):
-    """
-    Implements an interface to an lnd node.
-    """
-    def __init__(self, config_file=None, lnd_home=None, lnd_host=None,
-                 regtest=False):
+    """Implements the node interface for LND."""
+    def __init__(self, config_file: Optional[str] = None,
+                 lnd_home: Optional[str] = None,
+                 lnd_host: Optional[str] = None, regtest=False):
         """
         :param config_file: path to the config file
-        :type config_file: str
         :param lnd_home: path to lnd home folder
-        :type lnd_home: str
         :param lnd_host: lnd host of format "127.0.0.1:9735"
-        :type lnd_host: str
         :param regtest: if the node is representing a regtest node
-        :type regtest: bool
         """
         super().__init__()
-        self.config_file = config_file
+        if config_file:
+            self.config_file = config_file
+            self.config = settings.read_config(self.config_file)
+        else:
+            self.config_file = None
+            self.config = None
         self.lnd_home = lnd_home
         self.lnd_host = lnd_host
         self.regtest = regtest
@@ -123,11 +123,10 @@ class LndNode(Node):
                     'if lnd_home is given, lnd_host must be given also')
             lnd_host = self.lnd_host
         else:
-            config = settings.read_config(self.config_file)
-            cert_file = os.path.expanduser(config['network']['tls_cert_file'])
+            cert_file = os.path.expanduser(self.config['network']['tls_cert_file'])
             macaroon_file = \
-                os.path.expanduser(config['network']['admin_macaroon_file'])
-            lnd_host = config['network']['lnd_grpc_host']
+                os.path.expanduser(self.config['network']['admin_macaroon_file'])
+            lnd_host = self.config['network']['lnd_grpc_host']
 
         cert = None
         try:
@@ -313,7 +312,8 @@ class LndNode(Node):
             if c['private']:
                 self.total_private_channels += 1
 
-    def get_open_channels(self, active_only=False, public_only=False):
+    def get_open_channels(self, active_only=False, public_only=False) \
+            -> Dict[int, Dict]:
         """
         Fetches information (fee settings of the counterparty, channel
         capacity, balancedness) about this node's open channels and saves
@@ -431,6 +431,17 @@ class LndNode(Node):
             sorted(channels.items(), key=lambda x: x[1]['alias']))
         return sorted_dict
 
+    def get_channel_id_to_node_id(self, open_only=False) -> Dict[int, str]:
+        channel_id_to_node_id = {}
+        closed_channels = self.get_closed_channels()
+        open_channels = self.get_open_channels()
+        for cid, c in open_channels.items():
+            channel_id_to_node_id[cid] = c['remote_pubkey']
+        if not open_only:
+            for cid, c in closed_channels.items():
+                channel_id_to_node_id[cid] = c['remote_pubkey']
+        return channel_id_to_node_id
+
     def get_inactive_channels(self):
         """
         Returns all inactive channels.
@@ -465,6 +476,42 @@ class LndNode(Node):
         }
         return unbalanced_channels
 
+    def get_channel_fee_policies(self):
+        """
+        Gets the node's channel fee policies for every open channel.
+        :return: dict
+        """
+        feereport = self._rpc.FeeReport(lnd.FeeReportRequest())
+        channels = {}
+        for fee in feereport.channel_fees:
+            channels[fee.channel_point] = {
+                'base_fee_msat': fee.base_fee_msat,
+                'fee_per_mil': fee.fee_per_mil,
+                'fee_rate': fee.fee_rate,
+            }
+        return channels
+
+    def set_channel_fee_policies(self, channels: dict):
+        """
+        Sets the node's channel fee policy for every channel.
+        :param channels: channel point -> fee policy
+        """
+
+        for channel_point, channel_fee_policy in channels.items():
+            funding_txid, output_index = channel_point.split(':')
+            output_index = int(output_index)
+
+            channel_point = lnd.ChannelPoint(
+                funding_txid_str=funding_txid, output_index=output_index)
+
+            update_request = lnd.PolicyUpdateRequest(
+                chan_point=channel_point,
+                base_fee_msat=channel_fee_policy['base_fee_msat'],
+                fee_rate=channel_fee_policy['fee_rate'],
+                time_lock_delta=channel_fee_policy['cltv'],
+            )
+            self._rpc.UpdateChannelPolicy(request=update_request)
+
     @staticmethod
     def timestamp_from_now(offset_days=0):
         """
@@ -932,7 +979,19 @@ class LndNode(Node):
                 raise ConnectionRefusedError
         return succeeded_nodes
 
+    def pubkey_to_channel_map(self):
+        """
+        Determines a dict with node pubkeys this node has a channel with, which
+        maps to a list of all the channels with the node.
 
-if __name__ == '__main__':
-    node = LndNode()
-    print(node.get_closed_channels().keys())
+        :return: dictionary of pubkeys with list of channels as value
+        :rtype: dict[list]
+        """
+        channels = self.get_all_channels()
+
+        node_to_channel_map = defaultdict(list)
+
+        for c, cv in channels.items():
+            node_to_channel_map[cv['remote_pubkey']].append(c)
+
+        return node_to_channel_map
