@@ -14,8 +14,9 @@ from lndmanage.lib.exceptions import (
     RebalanceFailure,
     RebalancingTrialsExhausted,
 )
+from lndmanage.lib.fee_setting import FeeSetter, optimization_parameters
 from lndmanage.lib.info import Info
-from lndmanage.lib.listchannels import ListChannels
+from lndmanage.lib.listings import ListChannels, ListPeers
 from lndmanage.lib.lncli import Lncli
 from lndmanage.lib.node import LndNode
 from lndmanage.lib.openchannels import ChannelOpener
@@ -59,7 +60,6 @@ class Parser(object):
     def __init__(self):
 
         # figure out if lncli is available and determine path of executable
-        self.lncli = False
         self.lncli_path = None
 
         self.check_for_lncli()
@@ -128,9 +128,33 @@ class Parser(object):
             '--from-days-ago', default=60, type=int,
             help='time interval start (days ago)')
         parser_listchannels_hygiene.add_argument(
-            '--sort-by', default='rev_nfwd', type=str,
+            '--sort-by', default='rev_nfwd/a', type=str,
             help='sort by column (look at description)')
 
+        # cmd: listpeers
+        self.parser_listpeers = subparsers.add_parser(
+            'listpeers',
+            help='lists peers with extended information',
+            formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+        self.parser_listpeers.add_argument(
+            '--from-days-ago', default=60, type=int,
+            help='time interval start (days ago)')
+        self.parser_listpeers.add_argument(
+            '--sort-by', default='f/w', type=str,
+            help='sort by column (look at description)')
+        listpeers_subparsers = self.parser_listpeers.add_subparsers(
+            dest='subcmd')
+
+        # cmd: listpeers in
+        listpeers_subparsers.add_parser(
+            'in',
+            help="displays peers sorted by inward traffic")
+
+        # cmd: listpeers out
+        listpeers_subparsers.add_parser(
+            'out',
+            help="displays peers sorted by outward traffic")
+
         # cmd: rebalance
         self.parser_rebalance = subparsers.add_parser(
             'rebalance', help='rebalance a channel',
@@ -325,7 +349,7 @@ class Parser(object):
             help='info string can represent a node public key or a channel id')
 
         # cmd: lncli
-        if self.lncli:
+        if self.lncli_path:
             parser_lncli = subparsers.add_parser(
             'lncli',
             help='execute lncli')
@@ -374,21 +398,80 @@ class Parser(object):
             type=str,
             help='Comma-separated list of node pubkeys.')
 
+        # cmd: update-fees
+        self.parser_update_fees = subparsers.add_parser(
+            'update-fees',
+            description='Periodically running this command increases/decreases'
+                        'the fees on all channels by adapting them according to '
+                        'the forwarding demand in the last interval, which can '
+                        'be set by the parameter --from-days-ago. The fee optimization '
+                        'tries to keep a liquidity buffer for excess-demand times.'
+                        'Channels can be excluded via the config section'
+                        'excluded-channels-fee-opt.\n'
+                        'The command will prompt the fees it would set after a yes/no question.'
+                        "\n\n**Don't run this command too frequently (only once a week), "
+                        "otherwise you put strain on the network and the new"
+                        "fee policies won't reach end points like mobile phones"
+                        "and you will route less.**",
+            help='optimize the fees on your channels to increase revenue and to automatically rebalance',
+            formatter_class=argparse.RawDescriptionHelpFormatter)
+        self.parser_update_fees.add_argument(
+            '--cltv', type=int, default=optimization_parameters['cltv'],
+            help='CLTV time delta.')
+        self.parser_update_fees.add_argument(
+            '--min-base-fee-msat', type=int,
+            default=optimization_parameters['min_base_fee'],
+            help='The base fee cannot go lower than this.')
+        self.parser_update_fees.add_argument(
+            '--max-base-fee-msat', type=int,
+            default=optimization_parameters['max_base_fee'],
+            help='The base fee cannot go higher than this.')
+        self.parser_update_fees.add_argument(
+            '--min-fee-rate', type=float,
+            default=optimization_parameters['min_fee_rate'],
+            help='The fee rate cannot go lower than this.')
+        self.parser_update_fees.add_argument(
+            '--max-fee-rate', type=float,
+            default=optimization_parameters['max_fee_rate'],
+            help='The fee rate cannot go higher than this.'
+            'Half of this value is also used for initialization.'
+        )
+        self.parser_update_fees.add_argument(
+            '--init', action='store_true',
+            help='If set, uses half of max-fee-rate for fee rates.')
+        self.parser_update_fees.add_argument(
+            '--from-days-ago', type=int,
+            default=7,
+            help='Sets the number of days over which forwarding action is taken'
+                 'into account. This value should coincide with the fee '
+                 'update interval.')
+        self.parser_update_fees.add_argument(
+            '--target-forwarding-amount-sat', type=int,
+            default=optimization_parameters['r_t'],
+            help='The target for how much a channel should route per day.'
+                 'The value of this parameter will influence how much you earn in forwarding'
+                 'fees. If you set it too low, no forwardings will happen. If you set it too'
+                 'high, you sell your liquidity too cheaply.'
+                 'A good value could be half the amount you route in your most-income channel per day.')
+        self.parser_update_fees.add_argument(
+            '--reckless',
+            help='Update the fees without asking the user explicitly.',
+            action='store_true')
+
     def check_for_lncli(self):
         """
-        Looks for lncli in PATH or in LNDMANAGE_HOME folder. Sets self.lncli
-        and self.lncli_path. Executable in LNDMANAGE_HOME is prioritized.
+        Looks for lncli in PATH or in LNDMANAGE_HOME folder. Sets self.lncli_path.
+        Executable in LNDMANAGE_HOME is prioritized.
         """
         lncli_candidate = os.path.join(settings.home_dir, 'lncli')
 
         # look in lndmanage home folder after lncli
         if os.access(lncli_candidate, os.X_OK):
-            self.lncli = True
-            self.lncli_path = lncli_candidate
+            if self.lncli_path:
+                self.lncli_path = lncli_candidate
         # look in PATH
         else:
             path = spawn.find_executable('lncli')
-            self.lncli = True
             self.lncli_path = path
 
     def parse_arguments(self):
@@ -430,6 +513,32 @@ class Parser(object):
                 listchannels.print_channels_hygiene(
                     time_interval_start=time_from, sort_string=args.sort_by)
 
+        elif args.cmd == 'listpeers':
+            listpeers = ListPeers(node)
+            time_from = time.time() - args.from_days_ago * 24 * 60 * 60
+            time_to = time.time()
+            logger.info(
+                f"Forwardings from {args.from_days_ago} days ago"
+                f" to now are included.")
+            if not args.subcmd:
+                listpeers.print_all_nodes(
+                    time_interval_start=time_from,
+                    time_interval_end=time_to,
+                    sort_string=args.sort_by,
+                )
+            elif args.subcmd == 'in':
+                listpeers.print_all_nodes(
+                    time_interval_start=time_from,
+                    time_interval_end=time_to,
+                    sort_string='in',
+                )
+            elif args.subcmd == 'out':
+                listpeers.print_all_nodes(
+                    time_interval_start=time_from,
+                    time_interval_end=time_to,
+                    sort_string='out',
+                )
+
         elif args.cmd == 'rebalance':
             if args.target:
                 logger.warning("Warning: Target is set, this is still an "
@@ -447,6 +556,8 @@ class Parser(object):
                 logger.error(f"Too expensive: {e}")
             except RebalanceFailure as e:
                 logger.error(f"Rebalance failure: {e}")
+            except KeyboardInterrupt:
+                pass
 
         elif args.cmd == 'circle':
             rebalancer = Rebalancer(node, args.max_fee_rate, args.max_fee_sat)
@@ -525,6 +636,25 @@ class Parser(object):
                 )
             except Exception as e:
                 logger.info(e)
+        elif args.cmd == 'update-fees':
+            # overwrite default optimization parameters
+            optimization_parameters['cltv'] = args.cltv
+            optimization_parameters['min_base_fee'] = args.min_base_fee_msat
+            optimization_parameters['max_base_fee'] = args.max_base_fee_msat
+            optimization_parameters['min_fee_rate'] = args.min_fee_rate
+            optimization_parameters['max_fee_rate'] = args.max_fee_rate
+            optimization_parameters['r_t'] = args.target_forwarding_amount_sat
+
+            feesetter = FeeSetter(
+                node,
+                from_days_ago=args.from_days_ago,
+                parameters=optimization_parameters
+            )
+
+            feesetter.set_fees(
+                init=args.init,
+                reckless=args.reckless
+            )
 
 
 def main():
@@ -553,7 +683,7 @@ def main():
                     "You can type 'help' or 'exit'.")
         node = LndNode(config_file=config_file)
 
-        if parser.lncli:
+        if parser.lncli_path:
             logger.info("Enabled lncli: using " + parser.lncli_path)
 
         while True:
@@ -578,9 +708,13 @@ def main():
 
             # lncli execution
             if args_list[0] == 'lncli':
-                lncli = Lncli(parser.lncli_path, config_file)
-                lncli.lncli(args_list[1:])
-                continue
+                if parser.lncli_path:
+                    lncli = Lncli(parser.lncli_path, config_file)
+                    lncli.lncli(args_list[1:])
+                    continue
+                else:
+                    logger.info("lncli not enabled, put lncli in PATH or in ~/.lndmanage")
+                    continue
             try:
                 # need to run with parse_known_args to get an exception
                 args = parser.parser.parse_args(args_list)
